#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/parport.h>
#include <linux/ppdev.h>
#include <linux/ioctl.h>
#include <stdio.h>
#include <errno.h>
#include "gd.h"
#include "lcd.h"

void waitport(){
    long wl;
    for (wl=0; wl<WAITTIME; wl++);
}

void LcdSendByte(int pport,unsigned char data){
    unsigned char tmp;
//    printf("Data: \n");
    ioctl(pport, PPWDATA, &data);
    tmp=0^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    tmp=STROBE^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    waitport();
}

void LcdSendAddress(int pport, unsigned int addr) {
    unsigned char tmp;
//    printf("Address %i\n",addr);
    tmp=(addr) & 255;
    ioctl(pport, PPWDATA, &tmp);
    tmp=(A0)^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    tmp=(A0+STROBE)^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    waitport();
    tmp=(addr)>>8;
    ioctl(pport, PPWDATA, &tmp);
    tmp=(A1)^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    tmp=(A1+STROBE)^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    waitport();
}

void UploadConfig(int pport, char* mcsfile) {
    //Uploads the FPGA config so the FPGA can function
    //correctly as a controller.
    FILE *fd;
    int t;
    int x;
    int b;
    int c=0;
    const char hexchr[]="0123456789ABCDEF0123456789abcdef";
    char buffer[1024];
    unsigned char tmp;
    printf("Sending config file %s to FPGA...\n",mcsfile);
    fd=fopen(mcsfile,"r");
    while (!feof(fd)) {
	fgets(buffer,1024,fd);
//	printf("\nHex string: %s",buffer);
	if (buffer[0]==':' && (!strncmp(buffer+7,"00",2))) {
//	    printf("Is data.\n");
	    for (t=9; t<(strlen(buffer)-4); t+=2){
		b=((int)index(hexchr,buffer[t+1])-(int)hexchr) ;
	    	b+=(((int)index(hexchr,buffer[t])-(int)hexchr) )*16;
//		printf(" %X:",b);
		c++;
		for (x=0; x<8; x++) {
		    if ((b & 1)==0) tmp=0; else tmp=32;
		    ioctl(pport, PPWDATA, &tmp);
		    waitport();
		    tmp=tmp+128;
		    ioctl(pport, PPWDATA, &tmp);
		    waitport();
//		    tmp=tmp-128;
//		    ioctl(pport, PPWDATA, &tmp);
		    waitport();
		    b=b/2;
		}
	    }
	}
    }
    printf("%i bytes uploaded\n",c);
    fclose(fd);
//    exit(0);
}

void LcdLoadPic(int pport, gdImagePtr img){
    int x;
    int y;
    int z;
    unsigned char p;
    for (y=0; y<105; y++){
	LcdSendAddress(pport,y*256+1);
        for (x=0; x<639; x+=4) {
	    p=0;
	    if (gdImageGetPixel(img, x+0, y+104)==0) p+=1;
	    if (gdImageGetPixel(img, x+1, y+104)==0) p+=2;
	    if (gdImageGetPixel(img, x+2, y+104)==0) p+=4;
	    if (gdImageGetPixel(img, x+3, y+104)==0) p+=8;
	    if (gdImageGetPixel(img, x+0, y)==0) p+=16;
	    if (gdImageGetPixel(img, x+1, y)==0) p+=32;
	    if (gdImageGetPixel(img, x+2, y)==0) p+=64;
	    if (gdImageGetPixel(img, x+3, y)==0) p+=128;
	    LcdSendByte(pport,p);
        }
    }
}

void LcdClose(int pport) {
    unsigned char tmp;
    tmp=0^INVMSK;
    ioctl(pport, PPWCONTROL, &tmp);
    tmp=0;
    ioctl(pport, PPWDATA, &tmp);
    ioctl(pport, PPRELEASE);
    close(pport);
}

int LcdOpen(char* filename, char* fw) {
    int pport;
    int mode=IEEE1284_MODE_COMPAT;
    printf("Opening pport %s\n",filename);
    pport=open(filename, O_RDWR);
    if (pport==-1) {printf("Open: %s\n",strerror(errno)); return -1;}
    printf("Port mode: \n");
    if (ioctl(pport, PPSETMODE, &mode)) {printf("%s\n",strerror(errno)); return -1;}
    printf("Port claim: \n");
//    if (ioctl(pport, PPEXCL)) {printf("%s\n",strerror(errno)); return -1;}
    if (ioctl(pport, PPCLAIM)) {printf("%s\n",strerror(errno)); return -1;}
//    UploadConfig(pport,"lcdc_proto3_thahandy.mcs");
//    UploadConfig(pport,"lcdc1.mcs");
    UploadConfig(pport,fw);
    return pport;
}

