library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Lcdc_main is
    port (
        DataH: out STD_LOGIC_VECTOR (0 to 3);
        DataL: out STD_LOGIC_VECTOR (0 to 3);
        Clk: in STD_LOGIC;
--        Reset: in STD_LOGIC;        
        Cl1: out STD_LOGIC;
        Cl2: out STD_LOGIC;
        Din: out STD_LOGIC;
        M: out STD_LOGIC;
        Led: out STD_LOGIC;
        RamA: out STD_LOGIC_VECTOR (0 to 14);
        RamD: inout STD_LOGIC_VECTOR (0 to 7);        
				RamOE: out STD_LOGIC;
				RamCE: out STD_LOGIC;
				RamWE: out STD_LOGIC;
        ReqClk: in STD_LOGIC;				
				ReqAdIn: in STD_LOGIC_VECTOR (0 to 1);
        ReqDat: in STD_LOGIC_VECTOR(0 to 7);
        InvOut: out STD_LOGIC
    );
end Lcdc_main;

architecture Lcdc_main_arch of Lcdc_main is
component xcount is
    port (
        Enable: in STD_LOGIC;        
        Clk: in STD_LOGIC;
        Reset: in STD_LOGIC;        
        Carry: out STD_LOGIC;        
        Xpos: buffer STD_LOGIC_VECTOR (0 to 7)
    );
end component;
component ycount is
    port (
        Clk: in STD_LOGIC;
        Reset: in STD_LOGIC;        
        Ypos: buffer STD_LOGIC_VECTOR (0 to 6)
    );
end component;
component syncer is
    port (
        Xpos: in STD_LOGIC_VECTOR (0 to 7);
        Ypos: in STD_LOGIC_VECTOR (0 to 6);
        Cl1: out STD_LOGIC;
        Din: out STD_LOGIC;
				M: out STD_LOGIC    
    );
end component;
component requestreg2 is 
  port (Clk: in STD_LOGIC;
        ClrReq: in STD_LOGIC;
        ReqClk: in STD_LOGIC;
        ReqAdIn: in STD_LOGIC_VECTOR (0 to 1);        
        ReqDat: in STD_LOGIC_VECTOR (0 to 7);
        ReqAddr: out STD_LOGIC_VECTOR (0 to 14);
        ReqData: out STD_LOGIC_VECTOR (0 to 7);
        ReqValid: out STD_LOGIC);
end component;

component statemachine is 
  port (Clk: in STD_LOGIC;
        DispAddr: in STD_LOGIC_VECTOR(0 to 14);
        ReqAddr: in STD_LOGIC_VECTOR (0 to 14);
        ReqData: in STD_LOGIC_VECTOR (0 to 7);
        ReqValid: in STD_LOGIC;
        CL2: out STD_LOGIC;
        ClrReq: out STD_LOGIC;
        DispData: out STD_LOGIC_VECTOR (0 to 7);
        EnableNext: out STD_LOGIC;
        RamAddr: out STD_LOGIC_VECTOR (0 to 14);
        RamOE: out STD_LOGIC;
        RamWR: out STD_LOGIC;
        RamData: inout STD_LOGIC_VECTOR (0 to 7));
end component;
component LedBlinker is
    port (
        Clock: in STD_LOGIC;
        Led: out STD_LOGIC
    );
end component;

signal Xpos: STD_LOGIC_VECTOR (0 to 7);
signal Ypos: STD_LOGIC_VECTOR (0 to 6);
signal Xcarry: STD_LOGIC;
signal Reset: STD_LOGIC;
signal EnableCnt: STD_LOGIC;
signal ReqAddr: STD_LOGIC_VECTOR(0 to 14);
signal DispAddr: STD_LOGIC_VECTOR(0 to 14);
signal ReqData: STD_LOGIC_VECTOR(0 to 7);
signal ReqValid: STD_LOGIC;
signal DispData: STD_LOGIC_VECTOR(0 to 7);
signal ClrReq: STD_LOGIC;

begin
	States: Statemachine
		port map(Clk, DispAddr, ReqAddr, ReqData, ReqValid, CL2, ClrReq, DispData, EnableCnt, RamA, RamOE, RamWE, RamD);
  XCounter: XCount
  	port map(EnableCnt,Clk, Reset,Xcarry, XPos);
  YCounter: YCount
  	port map(Xcarry,Reset,Ypos);
  Sync: Syncer
  	port map(Xpos,Ypos,Cl1,Din,M);
  ReqReg: RequestReg2
  	port map(Clk, ClrReq, ReqClk, ReqAdIn, ReqDat, ReqAddr, ReqData, ReqValid);
--  LedBlnk: LedBlinker
--  	port map(Ypos(0), led);
  Reset<='0';
	RamCE<='0';
	DataH<=DispData(0 to 3);
	DataL<=DispData(4 to 7);	
	DispAddr(0 to 6)<=Ypos;
	DispAddr(7 to 14)<=XPos;
	led<=not ReqValid; 				--Testledje.
	--Testje: output bloksignaal om -12v te genereren
	InvOut<=XPos(2);
end Lcdc_main_arch;

--RAM indeling:
--Ram begint op (-1,0). Per byte 4 bits hoog beeld (uitgespreid in Xpos), 4 bits laag beeld.
--Virtuele beeldruimte: 1024X128

--Display-aansturing: Het display is in 2 gedeeltes opgedeeld, elk 104 (???) pixels hoog.
--Data wordt ingeklokt op de op- of neergaande (ben er nog steeds niet achter welke) flank van CL2: de 4 hoogste
--bits gaan naar het bovenste gedeelte van het LCD als 4 pixels naast elkaar, de 4 onderste naar de onderste.
--Cl1 reset de X-positie en zorgt dat het display de volgende lijn pakt.
--Din moet hoog gemaakt worden op het moment dat Cl1 geschopt wqordt voor de eerste lijn.
